from __future__ import annotations

import time
from typing import Any, Callable

import rich


def trace_time(method: Callable):
    def timed(*args: Any, **kwargs: Any):
        start_time = time.time()
        result = method(*args, **kwargs)
        runtime = int((time.time() - start_time) * 1000)
        if "log_time" in kwargs:
            name = kwargs.get("log_name", method.__name__.upper())
            kwargs["log_time"][name] = runtime
        else:
            rich.print(f"{method.__name__} took {runtime} ms")
        return result

    return timed


if __name__ == "__main__":

    @trace_time
    def func(a, b):
        for _ in range(int(1e6)):
            _ = a + b
        return a + b

    func(1, b=2)



import os as _os
import subprocess as _subprocess

import click as _click
import rich

from . import actions


class RunGroup(_click.Group):
    def get_command(self, ctx, cmd_name):
        rv = _click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        return None


@_click.command(cls=RunGroup, invoke_without_command=True)
@_click.pass_context
def cli(ctx):
    if ctx.invoked_subcommand is None:
        _click.echo(ctx.get_help())


@cli.command("init", help="init python project")
def init():
    rich.print("initializing python project agilely...")
    _dir = _os.path.dirname(_os.path.abspath(__file__))
    settings_dir = _os.path.join(_dir, "..", "settings")

    rich.print("generating ./vscode/settings.json...")
    vscode_path = _os.path.join(settings_dir, "vscode_settings.json")
    target_path = _os.path.join(".vscode", "settings.json")
    actions.copy_serializable(
        vscode_path, target_path, transform=actions.vscode_setting_transform, conflict="merge"
    )

    rich.print("generating ./.pre-commit-config.yaml...")
    precommit_path = _os.path.join(settings_dir, "pre-commit-config.yaml")
    actions.copy_serializable(precommit_path, ".pre-commit-config.yaml", conflict="merge")

    rich.print("generating ./pyproject.toml...")
    pyproject_path = _os.path.join(settings_dir, "pyproject.toml")
    actions.copy_serializable(
        pyproject_path, "pyproject.toml", transform=actions.pyproject_transform, conflict="merge"
    )

    rich.print("generating ./.gitignore.toml...")
    gitignore_path = _os.path.join(settings_dir, "gitignore")
    actions.copy_serializable(gitignore_path, ".gitignore", conflict="merge")

    rich.print("creating git information...")
    actions.git_init()

    rich.print("setting pre-commit hook...")
    hook_script = _os.path.join(_dir, "..", "scripts", "precommit_hook.sh")
    _subprocess.run(["bash", hook_script])



from __future__ import annotations

import os as _os
import subprocess as _subprocess
from typing import Callable, Optional

import rich
from open_serializer import Serializer, merge_dict_recursively

from . import utils


def git_init() -> None:
    if utils.check_is_git():
        rich.print("git already exists, skipping...")
    else:
        _subprocess.run(["git", "init"])


def vscode_setting_transform(settings: dict) -> None:
    which_black = _subprocess.run(["which", "black"], stdout=_subprocess.PIPE)
    which_black = which_black.stdout.decode("utf-8").strip()
    settings["python.formatting.blackPath"] = which_black


def pyproject_transform(settings: dict) -> None:
    import sys

    py_version = sys.version_info
    py_version = f"py{py_version[0]}{py_version[1]}"
    if py_version not in settings["tool"]["black"]["target-version"]:
        settings["tool"]["black"]["target-version"].append(py_version)


def copy_serializable(
    ref_path: str,
    target_path: str,
    injections: Optional[dict] = None,
    transform: Optional[Callable] = None,
    conflict: str = "merge",
) -> None:
    settings = Serializer.deserialize_object(ref_path)
    if _os.path.exists(target_path):
        if conflict == "skip":
            rich.print(f"pre-commit setting already exists at {target_path}, skipping...")
            return
        elif conflict == "merge":
            rich.print(f"pre-commit setting already exists at {target_path}, merging...")
            merge_dict_recursively(
                settings,
                Serializer.deserialize_object(target_path),
                force_replace=True,
                skip_duplicated_list=True,
            )
        elif conflict == "overwrite":
            rich.print(f"pre-commit setting already exists at {target_path}, overwriting...")
            _os.remove(target_path)
        else:
            raise ValueError(f"invalid conflict option: {conflict}")

    if injections is not None:
        settings.update(injections)

    if transform is not None:
        transform(settings)

    utils.require_folder(target_path)
    Serializer.serialize_object(settings, target_path)



import os as _os
import subprocess as _subprocess


def check_is_git() -> bool:
    check = _subprocess.call(
        ["git", "branch"], stderr=_subprocess.STDOUT, stdout=open(_os.devnull, "w")
    )
    if check != 0:
        return False
    else:
        return True


def require_folder(path: str) -> None:
    dir = _os.path.split(path)[0]
    if dir:
        if not _os.path.isdir(dir):
            _os.makedirs(dir)



from pathlib import Path

import setuptools

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setuptools.setup(
    name="agile-py",
    version="0.0.5",
    author="Jia-Yau Shiau",
    author_email="jiayau.shiau@gmail.com",
    description="agile python project utilities",
    url="https://github.com/Janus-Shiau/agile-py",
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            "agpy=agile_py.cli.cli:cli",
        ]
    },
    include_package_data=True,
    package_data={
        "": [
            "settings/*.toml",
            "settings/*.yaml",
            "settings/*.json",
            "settings/gitignore",
            "settings/*.md" "scripts/*.sh",
        ],
    },
    python_requires=">=3.7",
    license="Apache Software License (Apache-2.0)",
    install_requires=["click", "rich", "black", "isort", "ruamel.yaml", "toml", "open-serializer"],
    long_description=long_description,
    long_description_content_type="text/markdown",
)
